﻿; EnableWindow.ahk[UTF-8] (2025/01/25 22:00)
; for AutoHotKey v2.0.19 or later
;
; 指定のウィンドウの有効無効切替を行う
;    EnableWindow.exe Mode Target (ArrowStyle) (DenyStyle) (DispError) (Debug)
;       Mode
;          1/on/enable  : ウィンドウを有効化する
;          0/off/disable : ウィンドウを無効化する
;          2/sw/toggle   : 有効無効状態を現在値から変更する
;       Target
;          ウィンドウ文字列/ahk_exe フルパス/ahk_class クラスID など両端""でAutoHotKey
;          独自のウィンドウ指定記述、現在アクティブなウィンドウを指定する場合は"A"
;          https://ahkscript.github.io/ja/docs/v2/misc/WinTitle.htm
;          対象ウィンドウのウィンドウスタイルは WS_VISIBLE ビットが有効で WS_DISABLED ビットが無効の
;          ものとしています。複数ウィンドウに該当する場合はエラー表示し処理しません。
;          マッチを正規表現で記述したい時は先頭に"REGEX:"を書きます
;          AutoHotKeyV2の正規表現文法は下記参照
;          https://ahkscript.github.io/ja/docs/v2/misc/RegEx-QuickRef.htm
;          ex) "ahk_exe C:\Program Files\Mozilla Firefox\firefox.exe"
;          ex) "ahk_exe C:\Program Files\Vivaldi\Application\vivaldi.exe ahk_class Chrome_WidgetWin_1"
;       ArrowStyle
;          処理対象であるウィンドウスタイルのビット和(andをとって0以外になれば対象)の16進表記
;          省略された場合やdef/defaultと書かれた時は 0x10000000(WS_VISIBLE) が指定されたものとします
;          全てを対象外にしてしまう 0x00000000 は指定できません
;       DenyStyle
;          処理対象から除外するウィンドウスタイルのビット和(andをとって0になれば対象)の16進表記
;          省略された場合やdef/defaultと書かれた時は 0x08000000(WS_DISABLED) が指定されたものとします
;          全てを対象外にしてしまう 0xFFFFFFFF は指定できません
;       DispError
;          1/on/disp : 対象ウィンドウが見つからない時や複数ヒットした際のエラーダイアログを出す
;          [any] : 上記以外が指定されるか無指定ならエラー時にダイアログを出さない
;       DebugMode
;          1/on/debug : デバッグモードON
;          [any] : 上記以外が指定されるか無指定ならデバッグモードOFF
;

/* ウィンドウスタイル定義 */
DEFAULT_MASK    := 0x00C20000 ;//許可フラグが1つも立たない時に使われる必須フラグ初期値
                              ;//WS_BORDER + WS_DLGFRAME + WS_MINIMIZEBOX
WS_VISIBLE      := 0x10000000 ;//可視状態のウィンドウを作成
WS_DISABLED     := 0x08000000 ;//無効 (使用不能) なウィンドウを作成

/* 同時に複数実行を許可する */
#SingleInstance Off

#WinActivateForce
/* 非表示のウィンドウも捜索対象としておかないと仮想デスクトップで動作しない */
DetectHiddenWindows True


/* コマンドオプション数チェック */
if( A_Args.Length < 2 ){
	DispUsage()
}

/* モードとウィンドウ指定 */
iMode := A_Args[1]
strTarget := A_Args[2]
arg_regx := 2
if( StrUpper(SubStr(strTarget,1,6)) = "REGEX:" ){
	arg_regx := "RegEx"
	strTarget := SubStr(strTarget,7)
}

UniqueID := 0
iDisp := 0
iDebug := 0
iArrowStyle := 0
iDenyStyle := 0xFFFFFFFF

/* モードの扱い */
if( iMode = 1 or StrUpper(iMode) = "ON" or StrUpper(iMode) = "ENABLE" or StrUpper(iMode) = "ENABLED" ){
	iMode := 1
}else if( iMode = 0 or StrUpper(iMode) = "OFF" or StrUpper(iMode) = "DISABLE" or StrUpper(iMode) = "DISABLED" ){
	iMode := 0
}else if( iMode = -1 or iMode = 2 or StrUpper(iMode) = "SW" or StrUpper(iMode) = "TOGGLE" ){
	iMode := -1
}else{
	DispUsage()
}

/* 必須ウィンドウスタイルの扱い */
if( A_Args.Length >= 3 ){
	iArrowStyle := A_Args[3]
}
if( iArrowStyle = 0 or StrUpper(iArrowStyle) = "DEF" or StrUpper(iArrowStyle) = "DEFAULT" ){
	iArrowStyle := WS_VISIBLE
}else{
	if( IsNumber(iArrowStyle) ){
		iArrowStyle := Number(iArrowStyle)
	}
}

/* 除外ウィンドウスタイルの扱い */
if( A_Args.Length >= 4 ){
	iDenyStyle := A_Args[4]
}
if( iDenyStyle = 0xFFFFFFFF or StrUpper(iDenyStyle) = "DEF" or StrUpper(iDenyStyle) = "DEFAULT" ){
	iDenyStyle := 0x00000000
}else{
	if( IsNumber(iDenyStyle) ){
		iDenyStyle := Number(iDenyStyle)
	}
}


/* エラーダイアログの扱い */
if( A_Args.Length >= 5 ){
	iDisp := A_Args[5]
}
if( iDisp = 1 or StrUpper(iDisp) = "ON" or StrUpper(iDisp) = "DISP" ){
	iDisp := 1
}else{
	iDisp := 0
}

/* デバッグモードの指定 */
if( A_Args.Length >= 6 ){
	iDebug := A_Args[6]
}
if( iDebug = 1 or StrUpper(iDebug) = "ON" or StrUpper(iDebug) = "DEBUG" ){
	iDebug := 1
}else{
	iDebug := 0
}

/* 無効から有効に変える場合は対象ウィンドウが無効なので強制的にビットを操作 */
if( iMode ){
	iArrowStyle := iArrowStyle & ~WS_DISABLED
	iDenyStyle := iDenyStyle & ~WS_DISABLED
	if( iArrowStyle = 0x00000000 ){
		iArrowStyle := DEFAULT_MASK
	}
	if( iDenyStyle = 0xFFFFFFFF ){
		iDenyStyle := 0x00000000
	}
}

if( iDebug ){
	MsgBox( "PATH=" strTarget " REGX=" arg_regx)
}

/* タイトルマッチモードの設定(2=部分一致, "RegEx"=正規表現) */
SetTitleMatchMode(arg_regx)

/* フルパスに合致するウィンドウを列挙(非表示ウィンドウも対象としているため沢山ヒットする) */
listWin := WinGetList(strTarget)
idxWin := 0
findCount := 0

/* ウィンドウスタイルでマスク */
for hWin in listWin{
	idxWin := idxWin + 1
	try{
		style := WinGetStyle("ahk_id " hWin)
		if( iDebug ){
			MsgBox("Style=" "0x" SubStr("00000000" StrUpper(Format("{1:X}",style)),-8)
			. " Arrow=" "0x" SubStr("00000000" StrUpper(Format("{1:X}",iArrowStyle)),-8)
			. " Deny=" "0x" SubStr("00000000" StrUpper(Format("{1:X}",iDenyStyle)),-8))
		}
		if( style & iArrowStyle and not(style & iDenyStyle) ){
			findCount := findCount + 1
			UniqueID := hWin
		}
	}
}

if( UniqueID = 0 ){
	if( iDisp or iDebug ){
		MsgBox(strTarget " not found.")
	}
	ExitApp(1)
}

if( findCount != 1 ){
	if( iDisp or iDebug ){
		MsgBox(strTarget " has multiple windows.")
	}
	ExitApp(1)
}


try{
	WinSetEnabled( iMode, "ahk_id " UniqueID)
}catch TargetError as err{
	if( iDisp or iDebug ){
		MsgBox("WinSetEnabled Target Error.")
	}
	ExitApp(1)
}catch OSError as osr{
	if( iDisp or iDebug ){
		MsgBox("WinSetEnabled OS Error.")
	}
	ExitApp(1)
}

ExitApp(0)




DispUsage(){
	msg := 'Usage:`n'
	msg := msg 'EnableWindow.exe Mode Target (...options)`n`n'
	msg := msg '[Mode]`n'
	msg := msg '    1/on/enable = Enable Window`n'
	msg := msg '    0/off/disable = Disable Window`n'
	msg := msg '    2/sw/toggle = Switch en/dis Window`n'
	msg := msg '[Target]`n'
	msg := msg '   "window-title / REGEX:match-title-str`n'
	msg := msg '    ahk_exe full-path`n'
	msg := msg '    ahk_class class-name..."`n'
	msg := msg '[ArrowStyle]`n'
	msg := msg '    enabled WindowStyle Bits`n'
	msg := msg '    def/default = 0x10000000(WS_VISIBLE)`n'
	msg := msg '    ex) 0x16C70000`n'
	msg := msg '[DenyStyle]`n'
	msg := msg '    disabled WindowStyleBits`n'
	msg := msg '    def/default = 0x08000000(WS_DISABLED)`n'
	msg := msg '    ex) 0xE8000000`n'
	msg := msg '[DispError]`n'
	msg := msg '    1/on/disp = display ErrorDialog`n'
	msg := msg '[DebugMode]`n'
	msg := msg '    1/on/debug = Debug mode`n'
	MsgBox(msg)
	ExitApp(1)
}


